@@ -1,16 +1,18 @@
-#' Diagnosis of secondary progressive multiple sclerosis
-#' @description Diagnosis of conversion from relapsing-remitting multiple sclerosis (RRMS) to secondary progressive multiple sclerosis (SPMS), using the CORe definition, including Functional System Scores (FSS) of Expanded Disability Status Scale (EDSS). Diagnosis is based on clinical visit records, each record including entries for patient code, visit date, EDSS score, FSS, ambulation score, and days since most recent relapse.
+#' Identification of secondary progressive multiple sclerosis
+#' @description Identify conversion from relapsing-remitting multiple sclerosis (RRMS) to secondary progressive multiple sclerosis (SPMS), using the CORe definition, including Functional System Scores (FSS) of Expanded Disability Status Scale (EDSS). The identification of SPMS is based on clinical visit records, each record including entries for patient code, visit date, EDSS score, FSS, ambulation score, and days since most recent relapse.
+#' If a baseline EDSS score and corresponding FSS are not provided, these are determined as the first EDSS score and corresponding FSS recorded in the dataset, outside 30 days (the default) of a relapse.
+#' Following a relapse, the first EDSS score recorded in the dataset outside 30 days (the default) of a relapse, becomes the new baseline EDSS score.
+#' SPMS is sustained for the remainder of the follow-up, unless followed by two consecutive improvements in EDSS scores.
 #' @references Lorscheider J, et al. Brain 2016; 139 (9): 2395-2405.
-#' @param visits A data frame consisting of 12 columns: ID, dateEDSS, EDSS, FSpyr (pyramidal FSS), FScrbl (Cerebellar FSS), FSbstem (Brainstem FSS), FSsens (Sensory FSS), FSsph (Bowel Bladder FSS), FSvis (Visual FSS), FScereb (Cerebral FSS), FSamb (Ambulation score), daysPostRelapse (days since most recent relapse).
-#' @param minEDSS Minimum EDSS score to reach SPMS conversion.
+#' @param visits A data frame consisting of 22 columns: ID, dateEDSS, EDSS, FSpyr (pyramidal FSS), FScrbl (cerebellar FSS), FSbstem (brainstem FSS), FSsens (sensory FSS), FSsph (bowel bladder FSS), FSvis (visual FSS), FScereb (cerebral FSS), FSamb (ambulation score), dateBlineVisit, bEDSS (baseline EDSS), bFSpyr (baseline pyramidal FSS), bFScrbl (baseline cerebellar FSS), bFSbstem (baseline brainstem FSS), bFSsens (baseline sensory FSS), bFSsph (baseline bowel bladder FSS), bFSvis (baseline visual FSS), bFScereb (baseline cerebral FSS), bFSamb (baseline ambulation score), daysPostRelapse (days since most recent relapse).
+#' @param minEDSS Minimum EDSS score required to reach SPMS conversion.
 #' @param minFSpyr Minimum pyramidal FSS to reach SPMS conversion.
-#' @param tRelapse Minimum time in days from prior relapse to confirmation of EDSS progression.
+#' @param tRelapse Minimum time in days since the most recent relapse to EDSS assessment.
 #' @param tProgression SPMS confirmation period in days.
 #' @param tRegression Confirmation period for EDSS improvement in days.
-#' @param tRelProg Confirmation period (days) for rebaselining EDSS (after a relapse led to non-confirmed increase in EDSS).
+#' @param tRelProg Confirmation period (days) for re-baselining EDSS (after a relapse led to non-confirmed increase in EDSS).
 #' @importFrom stats aggregate
-#' @importFrom utils head tail
-#' @importFrom dplyr %>% group_by mutate
+#' @importFrom dplyr %>% group_by mutate bind_cols bind_rows
 #' @examples
 #' data(SampleData)
 #' output<-spmsDx(SampleData)
@@ -43,6 +45,7 @@ spmsDx <- function(visits, minEDSS = 4, minFSpyr = 2, tRelapse = 30, tProgressio
   SPDXlist <- NULL
 
   visits$dateEDSS <- as.Date(visits$dateEDSS, "%Y-%m-%d")
+  visits$daysPostRelapse <- abs(visits$daysPostRelapse)
 
   vis <- visits[which(!is.na(visits$EDSS)), ]
   vis <- vis[order(vis$ID, as.numeric(vis$dateEDSS)), ]
@@ -59,7 +62,7 @@ spmsDx <- function(visits, minEDSS = 4, minFSpyr = 2, tRelapse = 30, tProgressio
 
   for (p in 1:n) {
 
-    if ((p/n)==0.2||(p/n)==0.4|(p/n)==0.6|(p/n)==0.8|(p/n)==1.00) message("Processing patient #", p, "/", n, "  (", round(p*100/n, 2), "%)")
+    if ((p/n)==0.2|(p/n)==0.4|(p/n)==0.6|(p/n)==0.8|(p/n)==1.00) message("Processing patient #", p, "/", n, "  (", round(p*100/n, 2), "%)")
 
     pvisAll <- vis[which(vis$ID == patients$ID[p]), ]
 
@@ -69,17 +72,23 @@ spmsDx <- function(visits, minEDSS = 4, minFSpyr = 2, tRelapse = 30, tProgressio
       pvis <- pvisAll[min(which(pvisAll$FSNA == 0)) : nrow(pvisAll), ]
 
       pvis<-pvis[order(pvis$ID, pvis$dateEDSS),]
-      pvis$dateBlineVisit <- pvis$dateEDSS[1]
+
+      # Define bEDSS
+      if(all(is.na(pvis$bEDSS))) {
+        Visits.sn <- subset(pvis, pvis$daysPostRelapse>tRelapse | is.na(pvis$daysPostRelapse))
+        Visits.sn <- Visits.sn[order(Visits.sn$ID, as.numeric(Visits.sn$dateEDSS)), ]
+        pvis$dateBlineVisit <- as.Date(Visits.sn$dateEDSS[1])
+        for (k in 1:length(KurtzkeVars)) {
+          eval(parse(text = paste("pvis$b", KurtzkeVars[k], " <- Visits.sn[which(colnames(Visits.sn) == KurtzkeVars[k])][1, ]", sep = "")))
+        }
+        rm(k)
+      } else {
+        pvis$dateBlineVisit <- as.Date(pvis$dateBlineVisit, "%Y-%m-%d")
+      }
+
       pvis$daysPostBline <- as.numeric(pvis$dateEDSS - pvis$dateBlineVisit)
       pvis$daysPostBlineFinal <- as.numeric(max(pvis$daysPostBline))
 
-      # set baseline values
-
-      for (k in 1:length(KurtzkeVars)) {
-        eval(parse(text = paste("pvis$b", KurtzkeVars[k], " <- pvis[which(colnames(pvis) == KurtzkeVars[k])][1, ]", sep = "")))
-      }
-      rm(k)
-
       if (nrow(pvis) >= 3) {
 
         daysPostPrior <- c(0, diff(pvis$daysPostBline))
@@ -107,12 +116,12 @@ spmsDx <- function(visits, minEDSS = 4, minFSpyr = 2, tRelapse = 30, tProgressio
           pvisB$progression <- 0
           pvisB$progression <- ifelse(pvisB$bEDSS == 0.0, ifelse(pvisB$dEDSSvBline >= 1.5 & pvisB$EDSS >= minEDSS, 1, 0), ifelse(pvisB$bEDSS < 6.0, ifelse(pvisB$dEDSSvBline >= 1 & pvisB$EDSS >= minEDSS, 1, 0), ifelse(pvisB$dEDSSvBline >= 0.5 & pvisB$EDSS >= minEDSS, 1, 0)))
           pvisB<- pvisB %>%
-            group_by(ID) %>%
-            mutate(flag_0 = cumsum(progression == 1)) %>%
-            group_by(ID, flag_0) %>%
-            mutate(conseq = ifelse(progression == 0, sum(progression == 0), 0)) %>%
-            mutate(progression = ifelse(progression == 0 & conseq>1, 0, 1)) %>%
-            ungroup()
+            dplyr::mutate(flag_0 = cumsum(progression == 1)) %>%
+            dplyr::group_by(flag_0) %>%
+            dplyr::mutate(conseq = ifelse(progression == 0, sum(progression == 0), 0)) %>%
+            dplyr::ungroup() %>%
+            dplyr::mutate(progression = ifelse(progression == 0 & conseq>1, 0, 1))
+
           pvisB$regression <- 0
           pvisB$regression <- ifelse(pvisB$dEDSSvBline <= -0.5, 1, 0)
 
@@ -136,7 +145,7 @@ spmsDx <- function(visits, minEDSS = 4, minFSpyr = 2, tRelapse = 30, tProgressio
           # determine sustained EDSS progression
 
           rNoProg <- which(pvisBP$progression %in% 0)
-          rNoRel <- which( (pvisBP$daysPostRelapse >= tRelapse) | is.na(pvisBP$daysPostRelapse) )
+          rNoRel <- which( (pvisBP$daysPostRelapse > tRelapse) | is.na(pvisBP$daysPostRelapse) )
           for (v in 1:nrow(pvisBP)) {
             if (pvisBP$progression[v] == 0) {
               pvisBP$finalProgVisnum[v] <- NA
@@ -175,7 +184,6 @@ spmsDx <- function(visits, minEDSS = 4, minFSpyr = 2, tRelapse = 30, tProgressio
 
           pvisBPR <- pvisBP[1 : (which(pvisBP$progOrReg == 1)[1] - 1), ]
           pvisBPR <- pvisBPR[which(pvisBPR$dateEDSS != pvisBPR$dateBlineVisit[1]), ]
-          pvisBPR$postRelapseSig <- NA
           if (nrow(pvisBPR) >= 1 & pvisBP$progOrReg[1] != 1) {
             pvisBPR$postRelapseSig <- FALSE
             for (v in 1:nrow(pvisBPR)) {
@@ -283,7 +291,7 @@ spmsDx <- function(visits, minEDSS = 4, minFSpyr = 2, tRelapse = 30, tProgressio
           pvisBP$FSLeadConfirmed[1] <- length(which(daysFSlead >= tProgression)) > 0
           rm(daysFSlead)
 
-          # record whether an intervening relapse occured
+          # record whether an intervening relapse occurred
 
           pvisBP1 <- pvisBP[1, ]
           pvisBP1$interveningRelapse[which(is.na(pvisBP1$interveningRelapse))] <- FALSE
@@ -325,10 +333,14 @@ spmsDx <- function(visits, minEDSS = 4, minFSpyr = 2, tRelapse = 30, tProgressio
             EDSSMinRRMS <- min(pvisAll$EDSS[which(pvisAll$dateEDSS < newrow$dateEDSS)])
             EDSSMinSPMS <- min(pvisAll$EDSS[which(pvisAll$dateEDSS >= newrow$dateEDSS)])
 
-            newrow <- cbind(newrow, datePreDxFinal, EDSSPreDxFinal, visitsRRMS, visitsSPMS, daysVisitIntervalRRMSMax, daysVisitIntervalPreDxFinal, EDSSMinRRMS, EDSSMinSPMS)
-            rm(datePreDxFinal, EDSSPreDxFinal, visitsRRMS, visitsSPMS, daysVisitIntervalRRMSMax, daysVisitIntervalPreDxFinal, EDSSMinRRMS, EDSSMinSPMS)
+            newvars<-cbind(datePreDxFinal, EDSSPreDxFinal, visitsRRMS, visitsSPMS, daysVisitIntervalRRMSMax, daysVisitIntervalPreDxFinal, EDSSMinRRMS, EDSSMinSPMS)
+            newvars<-data.frame(newvars)
 
-            SPDXlist <- rbind(SPDXlist, newrow)
+            newrow <- dplyr::bind_cols(newrow, newvars)
+            rm(newvars, datePreDxFinal, EDSSPreDxFinal, visitsRRMS, visitsSPMS, daysVisitIntervalRRMSMax, daysVisitIntervalPreDxFinal, EDSSMinRRMS, EDSSMinSPMS)
+
+            SPDXlist<-data.frame(SPDXlist)
+            SPDXlist <- dplyr::bind_rows(SPDXlist, newrow)
             rm(newrow)
             break
           } else {
@@ -406,5 +418,4 @@ spmsDx <- function(visits, minEDSS = 4, minFSpyr = 2, tRelapse = 30, tProgressio
   return(SPDX)
 }
 globalVariables(c("ID", "dateEDSS", "EDSS", "FSpyr", "FSLeadConfirmed", "interveningRelapse", "daysPostRelapse", "dateBlineVisit", "bEDSS", "daysPostBline", "daysProgression", "dEDSSvBline", "date.obsFinal",
-                  "progression", "flag_0", "conseq", "pvisBPR", "pvisBPRN", "%>%", "group_by", "mutate", "ungroup"))
-
+                  "progression", "flag_0", "conseq", "pvisBPR", "pvisBPRN", "%>%", "mutate", "ungroup", "bind_cols", "bind_rows"))
